# Python Data structure

특징이 있는 정보는 어떻게 저장해야할까?

- 전화번호부
- 은행 번호표
- 서적 정보
- 창고에 쌓인 수화물을 역순으로 찾을 때?

이번 강의에서는 **파이썬에서 많이 사용되는 자료구조**에 대해서 배웁니다.

**자료구조**는 어떤 데이터를 저장할 때, 그 데이터에 특징에 따라 컴퓨터에 효율적으로 정리하기 위한 **데이터의 저장 및 표현 방식**을 이야기합니다. 어떤 데이터는 순서가 있다거나, 그 데이터의 나타내는 ID 값과 쌍을 이룬다던가 하는 등의 특징이 나타나게 됩니다. 일반적으로 사용되는 정수, 문자열등의 변수 타입보다는 **이러한** **특징들에 잘 맞는 형태로 데이터를 저장**하게 된다면 훨씬 **효율적으로 컴퓨터의 메모리를 사용**하고, **프로그래머가 코드를 작성하기에도 용이**하게 해줍니다.

중요한 내용들이 많은 챕터로 세심하게 이해하고 넘어가면 좋겠습니다.



# 1. Stack

- 나중에 넣은 데이터를 먼저 반환하도록 설계된 메모리 구조

- Last In First Out

- 데이터의 입력을 Push, 출력을 pop이라고 부른다. 



## 1.1. Stack with list object

- 리스트(스택)에 넣을땐 `.append()`

- 뽑을 땐 `.pop()`을 사용

  ```python
  >> a = [1,2,3,4,5]
  >> a.append(20)
  [1,2,3,4,5,20]
  
  >> a.pop()
  20
  >> a
  [1,2,3,4,5]
  
  >> a.pop()
  5
  >> a
  [1,2,3,4]
  ```

  > 주의할 점은 pop을 실행하는 행위 자체로 리스트의 마지막 값이 반환됨
  >
  > - 리스트의 남은 값들이 반환 되는 것이 아님

- 입력된 글자를 역순으로 출력할 때 유용



# 2. Queue

먼저 넣은 데이터를 먼저 반환하는 메모리 구조



## 2.1. queue with list object

- 스택과 마찬가지로 `append()`와 `pop()`을 이용
- `pop(0)`을 사용하면 마지막을 반환하는 것이 아니라 첫번째 인덱스를 반환



# 3. Tuple

- 값의 변경이 불가능한 리스트

- 선언시 `(, )`를 사용한다. 

- 리스트의 연산, 인덱싱, 슬라이싱 등을 동일하게 사용한다. 

  ```python
  >> t = (1,2,3,)
  >> type(t)
  tuple 
  
  >> len(t)
  3
  #값을 바꿀려고하면 에러.
  >> t[1]=5
  TypeError : 'tuple' object does not support item assignment
  ```

- 왜쓸까?

  - 프로그램 작동하는 동안 변경되지 않은 데이터의 저장
    - 학번, 이름, 우편번호 등
  - 사용자의 실수에 의한 에러를 사전에 방지



# 4. Set

- 데이터의 중복을 허용하지 않는 자료형

```python
>> s = set([1,2,3,4,1,2,3])
>> s
{1,2,3,4}

>> type(s)
set

>> a = {1,2,3,4,5}
>> type(a)
set

>> s = {2,3}
#원소 추가 
>> s.add(1)
>> s
{1,2,3}
#원소 삭제 1삭제
>> s.remove(1)
>> s
{2,3}

#[1,4,5,6,7] 추가
>> s.update([1,4,5,6,7])
>> s
{1,2,3,4,5,6,7}

>> s.discard(3)
>> s
{1,2,4,5,6,7}

# 모든 원소 삭제
>> s.clear()
>> s
set()
```



## 4.1. 집합의 연산

- 수학에서의 기본적인 연산은 지원한다.  
- `s1.union(s2)` : s1과 s2의 합집합
  - `s1 | s2` 도 가능
- `s1.intersection(s2)` : 교집합
  - `s1 & s2`
- `s1.difference(s2)` : 차집합



# 5. Dict

- 데이터를 저장할 때는 구분지을 수 있는 값을 함께 저장
  - 예)주민번호, 제품 모델 번호
- 구분을 위한 덴이터 고유 갑을 identifier 또는 key라고함
- key 값을 활용하여, 데이터 값(value)에 접근한다.

![image-20210120164526244](https://user-images.githubusercontent.com/38639633/105153523-e2c26e80-5b4b-11eb-9fc7-aa2e18322caf.png)

- ```python
  >> country_code={}# Dict생성, country_code= dict() 도가능
  >> country_code={＂America＂:1,＂Korea＂:82,＂China＂:86,＂Japan＂:81}
  >> country_code
  {＇America＇:1,＇China＇:86,＇Korea＇:82,＇Japan＇:81}
  
  >> country_code.items()# Dict데이터출력
  Dict_items([(＇America＇,1),(＇China＇,86),(＇Korea＇,82),(＇Japan＇,81)])
  
  >> country_code.keys()# Dict키값만출력
  Dict_keys(["America","China","Korea","Japan"])
  
  >> country_code["German"]=49# Dict추가
  >> country_code
  {'America':1,'German':49,'China':86,'Korea':82,'Japan':81}
  
  >> country_code.values()# DictValue만출력
  dict_values([1,49,86,82,81])
  ```



# 6. Collections

- list, tuple, dict에 대한 확장 자료구조

- 편의성, 실행 효율 등을 사용자에게 제공함

  ```python
  from collections import deque
  from collections import Counter
  from collections import OrderedDict
  from collections import defaultdict
  from collections import namedtuple
  ```

## 6.1. deque

- stack과 queue를 지원함
- rotate, reverse 등의 linked list의 특성을 지원한다. 
- 기존 list 형태의 함수를 모두 지원함
- list보다 효율적인 자료구조를 제공한다.
- 효율적 메모리 구조로 처리 속도 향상



## 6.2 OrderedDict

- dict와 달리 데이터를 입력한 순서대로 dict를 반환함
- 그러나 dict도 python 3.6 부터 입력한 순서를 보장하여 출력한다.
- 별 의미 없음



## 6.3. defaultdict

- dict type의 값에 기본 값을 지정, 신규값 생성시 사용하는 방법

  ```python
  >> d = dict()
  >> print(d['first'])
  KeyError : 'first'
      
  >> from collections import defaultdict
  >> d = defaultdict(object)
  >> d = defaultdict(lambda:0)
  >> print(d['first'])
  0
  ```

- 하나의 지문에 각 단어들이 몇 개나 있는지 세고 싶을 때

- Text-mining 접근법 - Vector Space Model 

  - 토큰을 dict에 넣고 토큰 갯수 카운트 할 수 있다. 

## 6.4. Counter

- Sequence type의 data element들의 갯수를 dict 형태로 반환
- 이건 패스 why? 잘아니깐



## 6.5. Namedtuple

- Tuple 형태로 ***데이터 구조체***를 저장하는 방법
- 저장되는 data의 variable을 사전에 지정해서 저장한다.

```python
>> from colletions import namedtuple

>> #basic example
>> Point = namedtuple('point',['x','y'])
>> Point
__main__.Point

>> p = Point(x = 11, y = 22)
>> p[0], p[1]
(11,22)

>> x, y = p
>> print(x, y)
>> print(p.x + p.y)
>> print(Point(x=11, y=22))
11 22
33
Point(x=11, y=22)
```







































































































