# Pythonic code

이번 강의에서는 파이썬 특유 문법을 의미하는 **pythonic code**에 대해 배웁니다.

앞서 우리는 파이썬의 가장 큰 장점 중 하나가 인간이 이해하고 쓰기 쉬운 언어라고 이야기를 했습니다. 파이썬의 이러한 특징을 가장 잘 살린 파이썬의 문법적 특징을 우리는 **pythonic code**라고 합니다.

pythonic code 는 앞서 우리가 살펴보았던 데이터 구조와 달리 특별히 모듈이나 함수가 존재하는 것은 아닙니다. 단지 앞에서 배운 str 이나 다양한 모듈들을 활용하여 **파이썬 특유의 문법**을 표현하는 것입니다. 파이썬 문법의 가장 큰 특징은 **짧고 이해하기 편하다**는 것 입니다. 코드의 수를 줄여서 비록 컴퓨터의 시간은 증가할 수 있지만, 사람의 시간은 아낄 수 있다는 장점이 있습니다.

추가적으로 python 2.x 버전에서 많이 썼던 **lambda, map, reduce** 와 난이도가 있는 파이썬 코딩을 위해 반드시 필요한 **asterisk** 의 활용에 대해서 배우도록 하겠습니다.



# Overview

- 파이썬 스타일의 코딩 기법
- 파이썬 특유의 문법을 활용하여 효율적으로  코드를 표현함
- 그러나 더 이상 파이썬'만'의 특징은 아님. 많은 언어들이 서로의 장점을 채용
- 고급 코드를 작성할 수록 더 많이 필요해진다.



# Contents

앞서 했던 내용과 중복인 것들은 pass한다.

- ~~split and join~~ 
- list comprehension
- ~~enumerate and zip~~
- lambda and map and reduce
- generator
- asterisk



# 1. List comprehension

- 기존 list 사용하여 간단히 다른 리스트를 만드는 방법

- 포괄적인 list, 포함되는 리스트라는 의미로 사용됨

- 파이썬에서 가장 많이 사용되는 기법 중 하나

- 일반적으로 for + append보다 속도가 빠르다.

  ```python
  >>> case_1 =["A","B","C"]
  >>> case_2 =["D","E","A"]
  ['AD','AE','AA','BD','BE','BA','CD','CE','CA']
  >>> result =[i+j for i in case_1 for j in case_2]
  >>> result
  ['AD', 'AE', 'AA', 'BD', 'BE', 'BA', 'CD', 'CE', 'CA']
  >>> result =[ [i+jfor i in case_1] for j in case_2]
  >>> result
  [['AD', 'BD', 'CD'], ['AE', 'BE', 'CE'], ['AA', 'BA', 'CA']]
  ```

  > 간단히 말해 리스트 내에서 for문 if문 등의 조건문을 사용하는 방식임.
  >
  > 깔끔해진다 코드가



# 2. enumerate & zip

- **enumerate :** list의 element를 추출할 때 번호를 붙여서 추출하는 방식. 

  ```python
  >>>for i, v in enumerate(['tic','tac','toe']):
  # list의 있는 index와 값을 unpacking한다.
  >>>...print(i,v)
  >>>...
  0tic
  1tac
  2toe
  
  >>>mylist=['a','b','c','d']
  >>>list(enumerate(mylist))# list의 있는 index와 값을 unpacking하여 list로 저장
  [(0,'a'),(1,'b'),(2,'c'),(3,'d')]
  
  >>>{i:j for i, j in enumerate('Artificial intelligence (AI), is intelligence demonstrated by machines, unlike the natural intelligence displayed by humans and animals.'.split())}
  # 문장을 list로 만들고 list의 index와 값을 unpacking하여 dict로 저장
  {0:'Artificial',1:'intelligence',2:'(AI),',3:'is',4:'intelligence',5:'demonstrated',6:'by',7:'machines,',8:'unlike',9:'the',10:'natural',11:'intelligence',12:'displayed',13:'by',14:'humans',15:'and',16:'animals.'}
  ```

- **zip** : 두개의 list 값을 병렬적으로 추출한다. 

  ```python
  >>> alist=['a1','a2','a3']
  >>> blist=['b1','b2','b3']
  >>> for a,b in zip(alist,blist):# 병렬적으로값을추출
  >>> ...print(a,b)
  >>> ...
  a1 b1
  a2 b2
  a3 b3
  
  >>> a,b,c=zip((1,2,3),(10,20,30),(100,200,300))#각 tuple의 같은 index 끼리묶음
  (1,10,100)(2,20,200)(3,30,300)
  
  >>>[sum(x) for x in zip((1,2,3),(10,20,30),(100,200,300))]
  # 각Tuple 같은index를묶어합을list로변환
  [111,222,333]
  ```

- enumerate 와 zip을 동시 적용한 모습

  ```python
  >>> alist=['a1','a2','a3']
  >>> blist=['b1','b2','b3']
  >>>
  >>> for i,(a,b) in enumerate(zip(alist,blist)):
  >>> ...print(i,a,b)# index alist[index] blist[index] 표시
  >>> ...
  0 a1 b1
  1 a2 b2
  2 a3 b3
  ```

# 3. Lambda & map & reduce

## 3.1. lambda

- 함수 이름 없이, 함수처럼 쓸 수 있는 익명함수(일회성 정의)
- 수학의 람다 대수에서 유래함
- PEP8에서는 lambda의 사용을 권장하지 않는다. 
  - `def f(x): return 2*x` **권장(O)**
  - `f = lambda x : 2*x` **권장(X)**

- **Lambda의 문제점**
  - 어려운 문법
  - 테스트의 어려움
  - 문서화 docstring의 지원 미비
  - 코드 해석의 어려움
  - 이름이 존재하지 않는 함수의 출현
  - ~~그러나 많이 쓴다...~~



## 3.2. map

- 두 개 이상의 list에도 적용 가능함. if filter도 사용 가능함.

  - ```python 
    list(map(lambda x:x**2 if x % 2==0 else x, ex))
    ```

- python3는 iteration을 생성한다. >> list를 붙여줘야 list 사용가능

- 실행 시점의 값을 생성, 메모리 효율적

  ![image-20210120184047598](https://user-images.githubusercontent.com/38639633/105156344-105ce700-5b4f-11eb-8d37-a806886370db.png)



## 3.3. reduce

- map과 달리 list에 똑같은 함수를 적용해서 통합

  ```python
  from functools import reduce
  print(reduce(lambda x, y: x+y, [1,2,3,4,5]))
  ```

  > ![image-20210120184226364](https://user-images.githubusercontent.com/38639633/105156487-426e4900-5b4f-11eb-909c-724feaa16839.png)

## 3.4. summary

- lambda, reduce는 python3에서 사용을 권장하지 않음
  - 직관이 떨어진다.
- legacy library나 다양한 머신러닝 코드에서는 여전히 사용중이다.



# 4. iterable object

- sequence형 자료형에서 데이터를 순서대로 추출하는 오브젝트

- **Characteristics**

  - 내부적 구현으로 `__iter__`와 `__next__`가 사용된다.

  - iter()와 next()함수로 iterable 객체를 iterator object로 사용한다. 

    ```python
    cities = ["Seoul", "Busan", "Jeju"]
    iter_obj= iter(cities)
    print(next(iter_obj))
    print(next(iter_obj))
    print(next(iter_obj))
    next(iter_obj)
    ```

# 5. generator

- iterable object를 특수한 형태로 사용해주는 함수

- element가 사용되는 시점에 값을 메모리에 반환한다. 

  - yield를 사용해 한번에 하나의 element만 반환한다. 

    ![image-20210120184732464](https://user-images.githubusercontent.com/38639633/105157104-fbcd1e80-5b4f-11eb-9e1d-10d3d4f063a1.png)

## 5.1. generator comprehension

- list comprehension과 유사한 형태로 generator형태의 list 생성

- generator expression 이라는 이름으로도 불려진다. 

- **[] 대신에 ()를 사용하여 표현**

  > ```python
  > >>> gen_ex = (n*n for n in range(500))
  > print(tpye(g))
  > ```

## 5.2. Why generator

- 일반적인 iterator는 generator에 반해 훨씬 큰 메모리 용량을 사용한다. 

  ![image-20210120185118229](https://user-images.githubusercontent.com/38639633/105157598-7dbd4780-5b50-11eb-82f5-dde67b92ab2c.png)

## 5.3. When generator

- list 타입의 데이터를 반환해주는 함수는 generator로 만들어라
  - 읽기 쉬운 장점, 중간 과정에서 loop이 중단될 수 있을 때
- 큰 데이터를 처리할 때는 generator expression 을 고려하라
  - 데이터가 커도 처리의 어려움의 없음
- 파일 데이터를 처리할때도 generator를 쓰자



# 6. Function passing arguments

- 함수에 입력되는 arguments의 다양한 형태
  - Keyword arguments
  - Default arguments
  - Variable-length arguments

## 6.1. Keyword arguments

- 함수에 입력되는 파라미터의 변수명을 사용, arguments를 넘긴다. 

  ```python
  def print_somthing(my_name, your_name):    
  	print("Hello {0}, My name is {1}".format(your_name,my_name))
      
  print_somthing("Sungchul","TEAMLAB")
  print_somthing(your_name="TEAMLAB", my_name="Sungchul")
  ```

  > 기본...중



## 6.2. Default arguments

- parameter의 기본값을 사용, 입력하지 않을 경우 기본값 출력

  ```python
  def print_somthing_2(my_name, your_name="TEAMLAB"):
  	print("Hello {0}, My name is {1}".format(your_name,my_name))
  
  print_somthing_2("Sungchul","TEAMLAB")
  print_somthing_2("Sungchul")
  ```

  > 굳이 안쓰면 그냥 디폴트로 사용되는걸 말하는거.



## 6.3. Variable-length asterisk

- 함수의 파라미터가 정해지지 않았다?
- 다항방정식? 마트 물건 계산 함수?
- **<u>가변인자</u>**(variable-length)를 사용
  - 개수가 정해지지 않은 변수를 함수의 parameter로 사용하는 법
  - Keyword arguments와 함께, argument추가가 가능
  - Asterisk(*) 기호를 사용하여 함수의 parameter를 표시함
  - 입력된 값은 tuple type으로 사용할 수 있음
  - 가변인자는 오직 한 개만 맨 마지막 parameter 위치에 사용가능

- 가변인자는 일반적으로 `*arg`를 변수명으로 사용

- 기존 parameter 이후에 나오는 값을 tuple로 저장함

  - ```python
    def asterisk_test(a,b,*args):
    	return a+b+sum(args)
    print(asterisk_test(1,2,3,4,5))
    ```

  - ```python
    def asterisk_test_2(*args):
    	x,y,z =args
    	return x, y, z
    print(asterisk_test_2(3,4,5))
    ```

- 키워드 가변인자

  - Parameter 이름을따로지정하지않고입력하는방법

  - asterisk(*) 두 개를 사용하여 함수의 parameter를 표시함

  - 입력된 값은 dict type으로 사용할 수 있음

  - 가변인자는 오직 한 개만 기존 가변인자 다음에 사용

    ```python
    >> def kwargs_test_1(**kwargs):
    >>     print(kwargs)
    >> def kwargs_test_2(**kwargs):
    >>     print(kwargs)
    >>     print("First value is {first}".format(**kwargs))
    >>     print("Second value is {second}".format(**kwargs))
    >>     print("Third value is {third}".format(**kwargs))
    
    >> def kwargs_test_3(one, two,*args,**kwargs):
    >>     print(one+two+sum(args))
    >>     print(kwargs)
        
    >> kwargs_test_3(3,4,5,6,7,8,9,first=3,second=4,third=5)
    
    42
    {'first': 3, 'second': 4, 'third': 5}
    ```

- Asterisk

  - 흔히 알고 있는 `*`를 의미한다.
  - 단순 곱셈, 제곱연산, 가변 인자 활용 등 다양하게 사용됨

  > ![image-20210120190714009](https://user-images.githubusercontent.com/38639633/105159706-ba8a3e00-5b52-11eb-8161-df247db4b4c6.png)

- asterisk –unpacking a container

  - tuple, dict 등 자료형에 들어가 있는 값을 unpacking
  - 함수의 입력값, zip 등에 유용하게 사용가능하다. 



































