# Python Object Oriented Programming

이번 강의에서는 **객체 지향 프로그래밍 언어**, **Object Oriented Programming(OOP)**에 대해서 배웁니다.

OOP는 프로그래밍 언어를 배우는 데 있어서 매우 중요한 개념입니다. 파이썬 자체도 OOP 형태로 구성되어 있기도 하지만, 파이썬 나오기 전에 대세 언어들이였던 **자바, C++, C#** 같은 언어들이 모두 OOP 기반의 언어들입니다. OOP를 배우는 것은 이전에 우리가 if 문이나 loop문을 배우듯이 프로그래밍 언어를 배우는 데 있어 가장 기본적인 개념이 되었습니다.

고난이도 프로그래밍을 위한 길로 생각하고 이번 챕터를 공부해보시기 바랍니다.

- 파이썬의 객체 지향 언어에 대해 이해하자
- 최근 모든 언어가 객체 지향을 '지향'하고 있기 때문에 알아두면 타 언어를 이해하기도 쉽다.



# 1. 객체지향 프로그래밍 개요

- Object-Oriented Programming, OOP
- 객체 : 실생활에서 일종의 물건
  - 속성과 행동을 가짐
- OOP는 이러한 객체 개념을 프로그램으로 표현가능하다. 
  - 속성은 변수, 행동은 함수로 표현됨
- OOP는 설계도에 해당하는 **클래스**와 실제 구현체인 **인스턴스**로 나뉨
  - 붕어빵틀 : Class
  - 붕어빵 : Instance



## 2. Objects in python

## 2.1. Class 구현하기

**Class 선언하기**

- class선언, object는 python3에서 자동 상속
- [naming]
  - 변수와 class명 함수명은 짓는 방식이 존재한다.
  - Snake_case : 띄어쓰기 부분에 "_"를 추가, 뱀처럼 늘여쓰기, 파이썬 함수 / 변수명에 사용
  - CamelCase : 띄어쓰기 부분에 대문자 사용, 낙타의 등 모양, 파이썬 Class명에 사용한다. 

## 2.2. Attribute 추가하기

- Attribute 추가는 `__init__`, `self`와 함께 사용

  - `__init__`은 객체 초기화 예약 함수

  - ```python
    class SoccerPlayer(object):
        def __init__(self, ㅌㅌㅌ)
    ```



**파이썬에서 `__`의 의미**

- `__`는 특수한 예약함수나 변수 그리고 함수명 변경(맨글링)으로 사용한다.
  - 예) `__main__`, `__add__` 등등...

 **예시**

```python
>> class SoccerPlayer(object):
>>     def __init__(self, name:str, position : str, back_num : int):
>>         self.name = name
>>         self.position = position
>>         self.back_num = back_num
>> abc = SoccerPlayer('son','FW','7')
>> park = SoccerPlayer('park','WF','13')
>> abc is park
False
```

## 2.3. method 구현하기

- 다양한 magic method
  - [https://corikachu.github.io/articles/python/python-magic-method](https://corikachu.github.io/articles/python/python-magic-method)
- 반드시 `self`를 인자로 추가해 줘야만 class 함수로 인정이 된다. 
- `__str__` `__add__`등 각 method의 역할을 알고 있으면 좋다. 
  - `__str__` : print 문의 출력을 표현



## 2.4. objects(instance) 사용하기

- object 이름 선언과 함께 초기값 입력하기

- 예)

  - ```python
    park = SoccerPlayer('park','WF','13')
    ```

  - park : 객체명

  - SoccerPlayer : class 명

  - `'park','WF','13' `:  `__init__`함수의 interface 초기값



# 3. OOP Implementation Example

- 구현 가능한 OOP 만들기 - notebook

  - note를 정리하는 프로그램
  - 사용자는 note에 뭔가를 적을 수 있다. 
  - note에는 content가 있고, 내용을 제거할 수 있다. 
  - 두개의 노트북을 합쳐 하나로 만들 수 있다. 
  - note는 notebook에 삽입된다.
  - notebook은 note가 삽입 될 때 페이지를 생성하며, 최고 300페이지까지 저장 가능하다
  - 300페이지가 넘으면 더 이상 노트를 삽입하지 못한다. 

- Note Class

  > ```python
  > class Note(object):
  >     def__init__(self, content= None):
  >         self.content= content
  >     def write_content(self, content):
  >         self.content= content
  >     def remove_all(self):
  >         self.content= ""
  >     def __add__(self, other):
  >         return self.content + other.content
  > 	def__str__(self):
  >         return self.content
  > ```

- NoteBook class

  > ```python
  > class NoteBook(object):
  >     def __init__(self, title):
  >         self.title = title
  >         self.page_number = 1
  >         self.notes = {}
  >     def add_note(self, note, page= 0):
  >         if self.page_number< 300:
  >             if page == 0:
  >                 self.notes[self.page_number] = note
  >                 self.page_number += 1
  >             else:
  >                 self.notes = {page: note}
  >                 self.page_number += 1
  >         else:
  >             print("Page가 모두 채워졌습니다.")
  >     def remove_note(self, page_number):
  > 		if age_number in self.notes.keys():
  >             return self.notes.pop(page_number)
  >         else:
  >             print("해당페이지는존재하지않습니다")
  >     def get_number_of_pages(self):
  >         return len(self.notes.keys())
  > ```



# 4. OOP Characteristics

## 4.1. Inheritance

- 부모 클래스로부터 속성과 Method를 물려받은 자식 클래스를 생성하는 것

- 예

  > ```python
  > class Person(object): # 부모클래스Person 선언
  > def __init__(self, name, age, gender):
  >   self.name = name
  >   self.age = age
  >   self.gender = gender
  > def about_me(self): # Method 선언
  >   print("저의이름은", self.name, "이구요, 제나이는", str(self.age), "살입니다.")
  > 
  > class Employee(Person): # 부모클래스Person으로부터상속
  > def __init__(self, name, age, gender, salary, hire_date):
  >   super().__init__(name, age, gender) # 부모객체사용
  >   self.salary= salary
  >   self.hire_date= hire_date# 속성값추가
  > 
  > def do_work(self): # 새로운메서드추가
  >   print("열심히일을합니다.")
  > 
  > def about_me(self): # 부모클래스 함수 재정의
  >   super().about_me() # 부모클래스 함수 사용
  > ```
>      print("제 급여는", self.salary, "원이구요, 제 입사일은", self.hire_date, " 입니다.")
>
>      ```
>      
>      ```

- `super().__init__(a,b,c)` : super는 자기가 상속한 부모 클래서의 객체를 사용할 수 있게끔 하는 함수이다.  

## 4.2. Polymorphism

- 같은 이름 메소드의 내부 로직을 다르게 작성한다. 

- Dynamic Typing 특성으로 인해 파이썬에서는 같은 부모 클래스의 상속에서 주로 발생함

- 중요한 OOP의 개념, 그러나 너무 깊이 알 필요 없다. 

  ![image-20210121205756315](https://user-images.githubusercontent.com/38639633/105348253-669f5800-5c2b-11eb-9db0-14651e761967.png)

  

## 4.3. Visibility

- 객체의 정보를 볼 수 있는 레벨을 조절하는 것
- 누구나 객체 안의 모든 변수를 볼 필요가 없음
  - 객체를 사용하는 사용자가 임의로 정보 수정
  - 필요 없는 정보에는 접근할 필요가 없음
  - 만약 제품으로 판매한다면? 소스의 보호

- **ENcapsulation**

  - 정보은닉
  - class를 설계할 때 글래스 간 간섭/정보공유의 최소화
  - 캡슐을 던지듯, 인터페이스만 알아서 정보를 사용할 수있다.

- **Visibility Example**

  - ```python
    class Product(object):
        pass
    class Inventory(object):
        def __init__(self):
            self.__items = [] # Private 변수로 선언 타객체가 접근 못함
            
        def add_new_item(self, product):
            if type(product) ==Product:
                self.__items.append(product)
                print('new item added')
            else:
                raise ValueError('invalid item')
        def get_number_of_items(self):
            return len9self.__items
    ```

    > `Product()`로 만들어진 객체가 아니면 접근이 불가능 하다. 
    >
    > `ma_inven = Inventory()`
    >
    > `ma_inven.add_new_item(Product())` # 추가해도됨
    >
    > `ma_inven.__items.append('aaa')` # 에러발생시켜야함

- **Visibility Example2**

  -  Inventory에 Product items의 접근이 허용하게 할 수도 있다. 
    - 데코레이터...



## 4.4. Decorator

### 4.4.1. first-class objects

- <u>일등함수 또는 일급 객체</u> 라고 변역된다

- 변수나 데이터 구조에 할당이 가능한 객체

- 파라미터로 전달이 가능 + 리턴 값으로 사용 :

  - **파이썬의 함수는 일급함수** : 예시 map(f, x) => 여기서 f는 함수지만 파라미터로 사용됨

- ```python
  def formula(method, argument_lst):
      return [method(value) for value in argument_lst]
  ```

  

### 4.4.2. inner function

- 함수 내에 또 다른 함수가 존재하는 방식

  ```python
  def print_msg(msg):
      def printer():
          print(msg)
      printer()
  print_msg('hello, python')
  ```

- **closures** : inner function을 return값으로 반환하는 방식

  ```python
  def print_msg(msg):
      def printer():
          print(msg)
  	return printer
  another = print_msg('hello, python')
  another()
  ```

  

### 4.4.3. decorator

- 위에서 등장한 복잡한 클로져 함수를 간단하게!

  ```python
  def star(func):
  	def inner(*args, **kwargs):
  		print("*"* 30)
  		func(*args, **kwargs)
  		print("*"* 30)
  	return inner
  @star
  def printer(msg):
  	print(msg)
  printer("Hello")
  ```

  > **<u>데코레이터</u>**(@)를 사용한 뒤 함수 명을 사용하면, 
  >
  > 예시처럼 다음에 나오는 함수가 star에 들어가게 된다 
  >
  > 출력 : 
  >
  > ```
  > ******************************
  > Hello
  > ******************************
  > ```

  































































