# File / Exception / Log Handling

이번 강의에서는 프로그램을 제대로 만들기 위해 알아야 하는 **예외 처리**와 **파일 다루기**에 대해서 배웁니다.

대부분의 경우 우리가 만드는 프로그램에는 우리가 **예상치 못한 \**오\**류**가 생기곤 합니다. 이를 해결하기 위해서 사전에 모든 경우를 다 지정해서 대비하는 경우도 있지만, 특정 상황에서 발생할 수 있는 **예외**를 포괄적으로 지정해서 대비하는 경우도 있습니다. 또, 실제 프로그램을 작성할 때는 **파일에 존재하는 데이터**를 사용하는 경우가 많습니다.

추가적으로 여러가지 **프로그램을 진행하면서 기록을 남기는** **로깅**에 대해서도 배웁니다. **로그 데이터**를 남기는 건 오늘날 데이터 기반의 애플리케이션 개발에서 매우 중요한 이슈가 되고 있습니다. **로그 데이터**를 설계하는 것이 하나의 분야로 자리잡고 있을 만큼 매우 관심을 받고 있습니다.

이번 장을 통해서 파이썬으로 데이터를 다루는 가장 기본적인 영역을 시작해보겠습니다.



**생각해보기**

- 프로그램 사용시 일어나는 오류

  - 주소를 입력하지 않고 배송요청
  - 저장도 안했는데 컴퓨터 전원이 나감
  - =>> 예상치 못한 일들이 생김

- 예외에는 두 가지 종류가 있다

  - 예측할 수 **<u>있는</u>** 예외 : 

    - 예측가능한 예외는 케이스를 분할하여 따로 정리해주면 된다. 

  - 예측할 수 **<u>없는</u>** 예외

    - 예측할 수 없는 예외의 경우 후속 조치 등 대처가 필요

      > ex) 
      >
      > 1. 없는 파일 호출 => 파일 없음을 알림
      > 2. 게임 이상 종료 => 게임 정보 저장 

    - 이를 **Exception Handling**을 통해 잘못된 상황에 대한 대처를 실시한다. 



# 1. Exception Handling

- try ~ except 문법

  ```python
  try : 
      예외 발생 가능 코드
  except <Exception Type>:
      예외 발생시 대응하는 코드
  ```

- Exceptions의 종류(Built-in Exception : 기본적으로 제공하는 예외)

  - IndexError
  - NameError
  - ZeroDivisionError
  - ValueError
  - FileNotFoundError

  

# 2. File Handling

## 2.1. 파일의 종류

- 기본적인 파일 종류로 text 파일과 binary 파일로 나눔

- 컴퓨터는 text 파일을 처리하기 위해 binary 파일로 변환시킴
  - Ex) pyc파일

- 모든 text 파일도 실제는 binary 파일

- ASCII/Unicode 문자열 집합으로 저장되어 사람이 읽을 수 있음



## 2.2. Binary 파일

- 컴퓨터만 이해할 수 있는 형태인 **이진(법)형식**으로 저장된 파일
- 일반적으로 메모장으로 열면 내용이 깨져 보임
- **Excel, word등**



## 2.3. Text 파일

- 인간도 이해할 수 있는 형태인 **문자열 형식**으로 저장된 파일
- 메모장으로 열면 내용 확인 가능
- 메모장에 저장된 파일, HTML, Python등



## 2.4. Python File Read

- 파이썬은 파일 처치를 위해 `open`키워드를 사용함

  ```python
  # 접근 모드
  # r : 읽기 모드
  # w : 쓰기 모드
  # a : 추가 모드
  f = open("<파일이름>", "접근 모드")
  contents = f.read()
  f.close()
  ```

  ```python
  # with 구문과 함께 사용하기
    with open("test.txt", "r") as f:
        # f.readlines() 파일 전체를 list로 반환
        # f.readline() 실행 시 마다 한 줄 씩 읽어오기
        contents = f.read()
  ```

  

## 2.5. Python File Write

```python
# with 구문과 함께 사용하기
  with open("test.txt", "w", encoding='utf8) as f:
      f.write('text')
```



## 2.6. Directory 다루기

- os 모듈을 사용
- shutil.copy : 파일 복사 함수
- `pathlib` 모듈을 사용해 path를 객체로 다룸

## 2.7. Pickle

- 파이썬의 객체를 영속화(persistence)하는 built-in 객체

- Ex) list, dictionary, Class 등 저장 가능

- 데이터, object등 실행 중 정보를 저장 -> 불러와서 사용

- 저장해야하는 정보, 계산 결과(모델)등 활용이 많음

  ```python
  import pickle
  #############################
  f = open("list.pickle", "wb") # write binary
  test = [1,2,3,4,5]
  pickle.dump(test, f)
  f.close()
  #############################
  f = open("list.pickle", "rb") # read binary
  test_pickle = pickle.load(f)
  print(test_pickle)
  f.close()
  # [1, 2, 3, 4, 5]
  ```

  

# 3. Logging Handling

## 3.1. Logging

- 프로그램이 실행되는 동안 일어나는 정보를 기록으로 남기기
- 유저의 접근, 프로그램의 Exception, 특정 함수의 사용
- Console 화면에 출력, 파일에 남기기, DB에 남기기 등
- 기록된 로그를 분석하여 의미있는 결과를 도출할 수 있음
- 실행시점에서 남겨야 하는 기록, 개발시점에서 남겨야하는 기록



## 3.2. print vs logging

- 기록을 print로 남기는 것도 가능
- 그러나 **Console 창에만 남기는 기록은 분석시 사용불가**
- 때로는 레벨별(개발, 운영)로 기록을 남길 필요도 있음
- 모듈별로 별도의 logging을 남길 필요도 있음
- 이러한 기능을 체계적으로 지원하는 모듈이 필요함



## 3.3. logging 모듈

- python의 기본 Log 관리 모듈

  ```python
  import logging
  logging.debug("debug")
  logging.info("info")
  logging.warning("warning")
  logging.error("error")
  logging.critical("critical")
  ```

  

## 3.4. logging level

- 프로그램 진행 상황에 따라 다른 Level의 Log를 출력함

- 개발 시점, 운영 시점마다 다른 Log가 남을 수 있도록 지원함

- DEBUG > INFO > WARNING > ERROR > Critical

- Log 관리시 가장 기본이 되는 설정 정보

- ![image](https://user-images.githubusercontent.com/38639633/105471320-f0a7f900-5cdd-11eb-929e-9da9a5d7ceb1.png)

- ```python
  import logging
  logger = logging.getLogger("main") # Logger 선언
  logger.basicConfig(level=logging.DEBUG)
  logger.setLevel(logging.DEBUG) # DEBUG level 부터 출력
  stream_hander = logging.FileHandler(
      "my_log.log", mode="w", encoding="utf8"
  ) # Logger의 output 방법 선언
  logger.addHandler(stream_hander) # Logger의 output 등록
  ```

- 실제 프로그램을 실행할 때 여러 설정이 필요

  - `configparser` - 파일에 저장
  - `argparser` - 실행시점에 출력

### 3.4.1. configparser

- 프로그램의 실행 설정을 file에 저장함

- Section, Key, Value 값의 형태로 설정된 설정 파일을 사용

- 설정파일을 Dict Type으로 호출 후 사용

  ```python
  '''
  'example.cfg'
  [SectionOne]
  Status: Single
  Name: D
  Value: Yes
  Age: 30
  '''
  import configparser
  config = configparser.ConfigParser()
  config.sections()
  config.read('example.cfg')
  config.sections()
  for key in config['SectionOne']:
      print(key)
      config['SectionOne']['status']
  ```



### 3.4.2. argparser

- Console 창에서 프로그램 실행시 Setting 정보를 저장함

- 거의 모든 Console 기반 Python 프로그램 기본으로 제공

- 특수 모듈도 많이 존재하지만(TF), 일반적으로 argparse를 사용

- Command-Line Option이라고 부름

- ```python
  import argparse
  parser = argparse.ArgumentParser(description='Sum two integers.')
  # 짧은 이름, 긴 이름, 표시명, Help 설명, Argument Type
  parser.add_argument('-a', "--a_value", dest='A_value', help='A integers', type=int)
  parser.add_argument('-b', "--b_value", dest='B_value', help='B integers', type=int)
  args = parser.parse_args()
  print(args)
  print(args.a)
  print(args.b)
  print(args.a + args.b)
  ```

  

## 3.5. Logging formatter

- Log 결과값의 format을 지정해줄 수 있음

  ```python
  formatter = logging.Formatter('%(asctime) %(levelname)s %(process)d %(message)s')  
  # 2018-01-18 22:47:04,385 ERROR 4410 ERROR occurred
  ```







