# Python data handling

이번 강의에서는 파이썬에서 다룰 수 있는 **CSV, 웹, XML, JSON 네 가지 데이터 타입**에 대해서 배웁니다.

**CSV** 파일 포맷은 어떠한 프로그래밍 언어든 데이터를 다루는 분야에서 가장 기본이 되는 타입형태입니다. 우리가 계속 써온 데이터 저장형태인 **엑셀의 텍스트 데이터 형태**로 이해하시면 됩니다. 매우 중요한 데이터 타입으로 파이썬으로 핸들링하는 것에 익숙해지는 것이 필요합니다.

다음으로 **웹**입니다. 우리 모두가 알고 있듯이 이미 웹은 우리 세상에서 가장 많은 정보를 제공하고 가장 많은 시간을 사용하고 있는 공간입니다. 우리는 현재 매일 쓰고 있는 웹에서 가장 많은 정보를 얻고 있습니다. 이번 챕터에서 우리는 이 정보를 어떻게 자동으로 확보할 것인지에 대해서 배웁니다. 먼저 기본적인 웹을 표현하는 가장 기본적인 언어인 **HTML**에 대해서 배웁니다. HTML은 **웹에서 나타나는 정보의 표현 방법 중 가장 대표적인 방법**으로 거의 모든 정보를 HTML 분석으로 얻을 수가 있습니다. 다음으로 그 HTML을 분석하기 위해 대표적인 방법인 **정규표현식(regex)**에 대해서 배우게 됩니다. 정규표현식은 텍스트 데이터 분석에 있어 가장 대표적인 분석 방법으로, HTML외에도 **다양한 유형의 텍스트에서 특정 정보를 뽑아내는데 사용**합니다.

마지막으로 데이터를 저장하는 다양한 포멧중 하나인 **XML(eXtensible Markup Languages)**과 **JSON(JavaScript Object Notation)**에 대해서 다루도록 하겠습니다. XML 프로그래밍 언어에서 데이터를 저장하고 불러오는 전통적인 파일포맷은 흔히 **레거시 시스템**(오래전에 구축된 시스템)에서 **raw파일을 저장하는 대표적인 포맷**입니다. **JSON**은 이와 달리 모바일이 활발히 사용되면서 사용되기 시작하는 저장 포맷으로 웹에서 많이 사용되는 **JavaScript의 문법을 활용하여 저장하는 포맷**입니다.



# 1. CSV data handling

> csv 데이터 핸들링은 추후 pandas를 다룰 때 많이 다루게 되므로 생략합니다.



# 2. Web

- 우리가 늘 쓰는 
- world wide web, 줄여서 웹이라고 부른다. 
- 원래는 물리학자들간 정보 교환을 위해 사용되었다. 
- 데이터 송수신을 위한 http 프로토콜 사용, 데이터를 표시하기 위해 HTML 형식을 사용



## 2.1. HTML(Hyper Text Markup Language)

- 웹 상의 정보를 구조적으로 표현하기 위한 언어

- 제목, 단락, 링크 등의 요소를 표시하기 위해  `Tag`를 사용한다.

- 모든 요소들은 꺽쇠 괄호 안에 둘러 쌓여있다.

  - ```html
    <title> Hello, World </title>
    ```

- 모든 HTML은 트리 모양의 포함 관계를 가진다. 

- 일반적으로 웹 페이지의 HTML 소스파일은 컴퓨터가 다운로드 받은 후 웹 브라우저가 해석 / 표시(rendering)



## 2.2. 왜 웹을 알아야 하는가?

- 정보의 보고, 많은 데이터들이 웹을 통해 공유된다. 
  - ex) 환율정보, 날씨정보 등..
- HTML 도 일종의 프로그램, 페이지 생성 규칙이 있다. 
  - 규칙을 분석하여 데이터의 추출이 가능
- 추출된 데이터를 바탕으로 하여 다양한 분석이 가능하다. 



## 2.3. 정규식 (regular expression)

- 정규 표현식 , regexp 또는 regex 등으로 불림
- 복잡한 문자열 패턴을 정의하는 문자 표현 공식
- 특정한 규칙을 가진 문자열의 집합을 추출
  - 010-0000-0000 : `^\d{3}\-\d{4}\-\d{4}$`
  - 209.252.101.40 : `^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$`

- 자막파일 연습

  > [https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/html/6-1.vtt](https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/html/6-1.vtt)

- 정규식 for HTML Parsing

  - 문법 자체가 매우 방대, 스스로 찾아서 하는 공부필요하다. 
  - 도움 주는 사이트 : 
    - [정규식 연습장](http://www.regexr.com/) 활용
    - 테스트하고 싶은 문서를 text란에 삽입
    - 정규식을 사용해서 찾아보기

- 기본 문법 #1

  - 문자 클래스 `[]`: `[`와 `]` 사이의 문자들과 매치라는 의미

    > 예) 
    >
    > [abc] : 해당 글자가 a,b,c 중 하나가 있다. 
    >
    > "a','before','deep', 'dud','sunset'.. 등

  - `-`를 사용 : 범위를 지정할 수 있음

    > 예)
    >
    > [a-zA-Z] : 알파벳 전체
    >
    > [0-9] : 숫자 전체 

- 기본문법 #2 - 메타문자

  - 정규 표현을 위해 원래 의미 X, 다른 용도로 사용되는 문자들

  - `. ^ $ * + ? { } [ ] \ | ( )`가 있다.

  - `.` : 줄바꿈 문자인 `\n`를 제외한 모든 문자와 매치

  - `*` : 앞에 있는 글자를 반복해서 나올 수 있음

    - tomor*ow tomorrow tomoow tomorrrrow ...

  - `+` : 앞에 있는 글자 최소 1회 이상 반복

  - `{m, n}`: 반복 횟수를 지정

    > "00:00:09" => [0-9]{2}:[0-9]{2}:[0-9]{2}

  - `?` : 반복 횟수가 1회

  - `|` : or의 의미

  - `^` : not의 의미

- 정규식 추출 연습 : pdf를 보고 연습해보기!

  - 원하는 페이지 `F12` 눌러서 소스 복사
  - 정규식 연습장에 복붙 후 연습 하기

- 정규식 in 파이썬

  - [https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/html](https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/html) 에서
  - `reg_ex-example_1.py`, `reg_ex-example_2.py` 
  - `reg_ex-example_3.py`, `reg_ex-example_4.py` 를 통해 연습해보기

# 3. XML(eXtensible Markup Language)

- 데이터의 구조와 의미를 설명하는 tag를 사용하여 표시하는 언어
- tag와 tag사이에 값이 표시되고, 구조적인 정보를 표현할 수 있음
- HTML과 문법이 비슷, 대표적인 데이터 저장방식
- 정보의 구조에 대한 정보인 스키마와 DTD 등으로 정보에 대한 정보(**메타정보**)가 표현되며, 용도에 따라 다양한 형태로 변경이 가능하다.
- XML은 컴퓨터(예 : PC to 스마트폰)간에 정보를 주고받기 **매우 유용한 저장방식**으로 쓰이고 있다. 



## 3.1. XML Parsing in Python

- XML도 HTML과 같이 구조적 markup 언어이다
- 따라서 정규표현식으로 파싱이 가능
- 하지만 더 손쉬운 도구들이 개발되어 있다. 
- 가장 많이 쓰이는 parser는 **<u>beautifulsoup</u>**이다.



## 3.2. beautifulsoup

> [https://www.crummy.com/software/BeautifulSoup](https://www.crummy.com/software/BeautifulSoup)

- lxml과 html5lib과 같은 Parser를 사용함

- 속도는 상대적으로 느리나 간편히 사용할 수 있다. 

- conda 가상환경으로 lxml과 beautifulsoup 설치

  ```python
  activate <가상환경이름>
  conda install lxml
  conda install -C anaconda beautifulsoup4=4.5.1
  ```

- 모듈 사용

  - ```python
    from bs4 import BeautifulSoup
    
    #객체생성
    soup = BeautifulSoup(books_xml, 'lxml')
    #Tag 찾는 함수 find_all 생성
    	#find_all : 정규식처럼 해당 패턴을 모두 반환한다. 
    soup.find_all("author")
    ```

- 예제는 여기

  > [https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/xml](https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/tree/main/codes/python/data_handling/xml)



# 4. Json(JavaScript Object Notation)

- 자바스크립트 오브젝트 노테이션

- 원래 웹 언어인 Java Script의 데이터 객체 표현 방식

- 간결성으로 기계 / 인간이 모두 이해하기 편함

- 데이터 용량이 적고, Code로의 전환이 쉬움

- 이로인해 XML의 대체제로 많이 활용되고 있음

- Python의 Dict 타입과 동일한 형식

- XML을 간편하게 Json으로 변환 가능

  > ![image](https://user-images.githubusercontent.com/38639633/105483768-acbcf000-5ced-11eb-90f7-cf987d76b434.png)



## 4.1. Json in python

- json 모듈을 사용하여 손쉽게 파싱 및 저장 가능
- 데이터 저장 및 읽기는 dict타입과 상호 호환 가능
- 웹에서 제공하는 API는 대부분 정보 교환시 JSON 활용
- 페이스북, 트위터, Github 등 거의 모든 사이트 
- 각 사이트마다 Developer API의 활용법을 찾아 사용
- 파이썬의 key, value처럼 key값으로 value에 접근한다. 





































































 