**numpy**는 파이썬으로 진행되는 모든 데이터 분석과 인공지능 학습에 있어 가장 필수적으로 이해해야 하는 도구입니다.

numpy는 **Num**erical **Py**thon의 약자로 일반적으로 과학계산에서 많이 사용하는 선형대수의 계산식을 파이썬으로 구현할 수 있도록 도와주는 라이브러리입니다. 기존 가장 대중적으로 쓰이던 도구는 MATLAB인데 MATLAB의 역할을 파이썬의 도구로 생각하면 좋습니다.

numpy는 numpy 자체로도 많이 사용되지만 이후에 사용되는 SciPy나 Pandas의 base 객체로도 사용되며 numpy에서 사용되는 다양한 코드 표현법을 그대로 pytorch와 tensorflow에 사용하는 경우가 많아 numpy의 활용법은 반드시 알아둘 필요가 있습니다.

# 1. Numpy(array)

- Numerical Python 의 약자임

- 고성능 과학 계산용 패키지

- Matrix와 Vector와 같은 Arrary 연산의 사실상의 표준

- 한글로 넘파이로 주로 통칭

- ~~누군가는 넘피/늄파이라고도 말함~~

- 넘파이의 특징

  - 일반 List에 비해 빠르고, 메모리 효율적
  - 반복문 없이 데이터 배열에 대한 처리를 지원한다
  - 선형대수와 관련된 다양한 기능 제고
  - C, C++, 포트란 등의 언어와 통합 가능하다. 

- 설치 : 

  ```python
  conda install numpy
  ```

## 1.1. array creation

```python
test_array = np.array([1,2,3,4], float)
print(test_array)
type(test_array[3])
```

- numpy는 np.array 함수를 활용 배열을 생성한다. 

  - ndarray 객체

- numpy는 하나의 데이터 타입만 배열에 넣을 수 있다. 

  - 리스트의 원소는 다양한 타입의 데이터가 들어갔었음
  - 하지만 array는 다르다. 

- List와 가장 큰 차이점 -> dynamic typing not supported(동적 타입 지원 X)

- C의 Array를 사용하여 배열을 생성한다. 

- [코드 예제](https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/blob/main/codes/numpy/1_numpy_ndarray.ipynb)

-  ![image-20210125105626760](https://user-images.githubusercontent.com/38639633/105793710-f7c85300-5fcc-11eb-8d70-ca156abd6790.png)

  >  array는 주소값이 없고, 다이렉트로 들어가기 때문에 연산이 좋아진다.

- ![image-20210125105829287](https://user-images.githubusercontent.com/38639633/105793728-ff87f780-5fcc-11eb-9d01-c1432e5ced3d.png)

  > 메모리의 크기도 일정하기 때문에 공간을 계산하기도 수월하다. 

## 1.2. Array shape

- array의 RANK에 따라 불리는 이름이 있음

  - `rank : 0` : scalar
  - `rank : 1` : vector
  - `rank : 2` : matrix
  - `rank : 3` : 3-tensor
  - `rank : n`  : n-tensor

- shape의 형태는 rank에 따라 다르다.

  - 1-rank : (3,)

  - 2-rank : (3,4)

  - 3-rank : (4,3,4)

    ![image-20210125110849548](https://user-images.githubusercontent.com/38639633/105794050-92c12d00-5fcd-11eb-8772-ed80d4b8c5ed.png)

    > 앞의 4는 batch라고도 한다. (in ML)

## 1.3. Array dtype

- ndarray의 single element가 가지는 data type

- 각 element가 차지하는 memory의 크기가 결정됨

  - ex) 

    ```python
    np.array([[1,2,3],[2,3,4]], dtype = int) # 데이터 타입 : 정수
    np.array([[1,2,3],[4,5, '5','6'], dtype = np.float32) # 데이터 타입 : float선언
    ```



## 1.4. Array nbytes

- nbytes : ndarray object의 메모리 크기를 반환하는 함수

  ```python
  >>> np.array([[1,2,3],[2,3,4]], dtype = float32).nbytes
  32bits = 4bytes => 6 * 4bytes = 24bytes
  ```



# 2. Handling shape

## 2.1. reshape

- reshape : Array의 shape의 크기를 변경함, element의 갯수는 동일하다. 
  - ex) (2, 4) -> (8, )
- `-1` in reshape : -1을 사용하면, 다른 size에 맞게 조정된다. 
  - ex) (2, 4) => (-1, 2) = (4, 2)
  - ex2) (2,2,2,2) => (-1, 2) = (8, 2)
- [코드 예제](https://github.com/BoostcampAITech/lecture-note-python-basics-for-ai/blob/main/codes/numpy/2_numpy_reshape.ipynb)



## 2.2. flatten

- reshape과 비슷하다
- 단순히 "일자로" 펴기 위한 함수
  - (2, 2, 4) => flatten => (16, )



## 2.3. indexing & slicing

- List와 달리 이차원 배열에서 [0,0] 표기법을 제공한다. 

- matrix일 경우 앞은 row, 뒤는 column을 의미한다. 

  ```python
  >> a = np.array([[1, 2, 3], [4.5, 5, 6]], int)
  >> print(a)
  >> print(a[0,0]) # Two dimensional array representation #1
  >> print(a[0][0]) # Two dimensional array representation #2
  [[1, 2, 3], 
   [4.5, 5, 6]]
  1
  1
  
  >> a[0,0] = 12 # Matrix 0,0 에 12 할당
  >> print(a)
  >> a[0][0] = 5 # Matrix 0,0 에 12 할당
  >> print(a)
  [[12, 2, 3], 
   [4.5, 5, 6]]
  [[5, 2, 3], 
   [4.5, 5, 6]]
  ```

- list와 달리 행과 열 부분을 나눠서 slicing이 가능하다. 

  - 각 행,열별로 따로 슬라이싱

- matrix의 부분집합을 추출할 때 유용함

- 주의
  
  - 인덱싱할 때, index로 접근하는 것과 slicing하는 것의 차이를 구분해야함
- ![image-20210125112941896](https://user-images.githubusercontent.com/38639633/105794054-948af080-5fcd-11eb-9b7e-50ba425f45d9.png)

## 2.4. creation function

**arange**

- array의 범위를 지정하여, 값의 list를 생성하는 명령어

  ```python
  >> np.aragne(0, 5, 0.5)
  array([0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5])
  ```



**zeros** - 0으로 가득찬 ndarray 생성

- `np.zeros(shape, dtype, order)`



**ones** - 1로 가득찬 ndarray 생성

- `np.ones(shape, dtype, order)`



**empty** - shape만 주어지고 비어있는 ndarray 생성

- meeory initialization이 되지 않는다. : 공간만 생성되기 때문이다.
- `np.empty(shape = (10,), dtype = np.int8)`



**something_like**

- 기존 ndarray의 shape 크기만큼 1, 0 또는 empty array를 반환한다.

  ```python
  >> test_mat = np.arage(30).reshape(5,6)
  >> np.ones_like(test_mat)
  array([[1,1,1,1,1,1],
        [1,1,1,1,1,1],
        [1,1,1,1,1,1],
        [1,1,1,1,1,1],
        [1,1,1,1,1,1]])
  ```



**identity**

- 단위 행렬(i 행렬)을 생성한다. 
  - `np.identity(n = 3, dtype = np.int8)`



**eye**

- 대각선인 1인 행렬, k값의 시작 index의 변경이 가능

  - identity와 다르다.

- ```python
  np.eye(N=3, <=5, dtype=np.int8)
  array([[1,0,0,0,0],
        [0,1,0,0,0],
        [0,0,1,0,0]], dtype = int8)
  ```



**diag**

- 대각행렬의 값을 추출한다. 
- `np.diag(matrix, k=1)` : k => start index



## 2.5. operation functions

**sum, axis, mean & std**

- 기초적인 operation이므로 이는 생략한다. 



**concatenate**

- numpy array를 합치는 함수

- `np.vstack`과 `np.hstack`을 이용하여 붙일 수 있다. 

  ![image-20210125121404292](https://user-images.githubusercontent.com/38639633/105794057-95bc1d80-5fcd-11eb-81dd-4f4789870f37.png)

- `np.concatenate((array1, array2), axis = 0, 1)`

  - 마찬가지로 array를 붙이는 함수
  - axis를 설정함으로써 방향을 정할 수 있다. 



## 2.6. array operations

**Operations b/t arrays**

- array간의 기본적인 사칙연산을 지원함

  

**Element-wise operations**

- array간의 shape이 같을 때 일어나는 연산

- element-wise하게 연산이 일어난다. 

  - 같은 자리에 위치한 element끼리 연산하는 방식
  - ![image-20210125121757345](https://user-images.githubusercontent.com/38639633/105794063-9785e100-5fcd-11eb-9119-0de0ea95a4ca.png)

  

**Dot product**

- 원래 알고 있던 수학에서의 행렬 연산을 의미한다.
- ![image-20210125121840641](https://user-images.githubusercontent.com/38639633/105794065-981e7780-5fcd-11eb-945d-69c974aaa477.png)



**transpose**

- transpose 또는 T attribute를 사용한다. 
- 행렬 전치



**broadcasting**

- shape이 다른 배열간의 연산을 지원하는 기능

  - ex)

    ```python
    >> np.array([[1,2,3],[4,5,6]], float) + 3
    array([[4., 5., 6.],
          [7., 8., 9.]])
    ```

- Scalar - vector 간의 연산 외에도 vector - matrix 간의 연산도 지원한다. 

  - ex)

    ```python
    >> test_matrix = np.arange(1, 13).reshape(4,3)
    >> test_vector = np.arange(10,40,10)
    >> test_matrix + test_vector
    array([[11, 22, 33],
          [14, 25, 36],
          [17, 28, 39],
          [20, 31, 42]])
    ```

    > ![image-20210125122312018](https://user-images.githubusercontent.com/38639633/105794067-994fa480-5fcd-11eb-9d64-0147da81c8f1.png)

- **numpy performance #1**

  - timeit : jupyter 환경에서 코드의 퍼포먼스를 체크하는 함수

  - `%timeit`

    ```python
    %timeit sclar_vector_product(scalar, vector) # for loop을 이용한 성능 
    %timeit [scalar * value for value in range(iternation_max)] # list comprehension을 이용한 성능 
    %timeit np.arange(iternation_max) * scalar # numpy를 이용한 성능
    ```

    

- **numpy performance #2**

  - 일반적으로 속도는
    - for loop < list comprehension < numpy
  - 1억 loop를 돌 때, 약 4배의 성능 차이를 보인다. 
  - numpy는 C로 구현되어 있어, 성능을 확보하는 대신 파이썬의 가장 큰 특징인 dynamic typing을 포기한다.
  - 대용량 계산에서는 가장 흔히 사용됨
  - Concatenate 처럼 계산이 아닌, 할당에서는 연산속도의 이점이 없다.



# 3. Comparisons

**All & Any**

- 데이터의 전부, 혹은 일부가 조건에 만족 여부 반환
- `np.any(a>5)`



**np.where**

- 해당 조건에 만족하는 array의 원소가 속한 index를 반환한다. 



**argmax & argmin**

- array내 최대값 또는 최소값의 index를 반환한다.
- axis 기반의 반환
- tensor의 softmax에 자주 사용된다.

